'use strict';
//Variables globales
var Alexa = require('alexa-sdk');
var states = {
    STARTMODE: '_STARTMODE',                
    ASKMODE: '_ASKMODE',
	RESTARTMODE: '_RESTARTMODE'     
};

//Nodos de decisión
var nodes = [{ "node": 1, "message": "Would you like to play a game?", "yes": 2, "no": 3 },
             { "node": 2, "message": "I haven't even told what the game is about. Are you sure?", "yes": 4, "no": 3 },
             { "node": 3, "message": "Why not? Are you afraid?", "yes": 10, "no": 5 },
             { "node": 4, "message": "It could be dangerous. Still sure?", "yes": 6, "no": 10 },
             { "node": 5, "message": "So, you are a very brave person then.", "yes": 11, "no": 10 },
             { "node": 6, "message": "Cool, let's play. Do you like cats?", "yes": 12, "no": 13 },
//Nodos finales
             { "node": 10, "message": "Well, my game is for brave people, you can't play anyway. Bye.", "yes": 0, "no": 0 },
             { "node": 11, "message": "Watch out, we got a badass over here. I don't play with liars. Bye.", "yes": 0, "no": 0 },
             { "node": 12, "message": "WRONG. You lose.", "yes": 0, "no": 0},
             { "node": 13, "message": "Din din din. You win. Cats sucks.", "yes": 0, "no": 0 },
];
//Mensajes a usar
var welcomeMessage = "Welcome to decision tree, are you ready to start?";
var repeatWelcomeMessage = "Say yes to start the game or no to quit.";
var promptToStartMessage = "Say yes to continue, or no to end the game.";
var promptToSayYesNo = "Say yes or no to answer the question.";
var playAgainMessage = "Do you want to play again?";
var helpMessage = "I will ask you some yes/no questions. Want to start now?";
var goodbyeMessage = "Ok, see you next time!";
var speechNotFoundMessage = "Could not find speech for node";
var nodeNotFoundMessage = "In nodes array could not find node";
var utterancePlayAgain = "play again";

//Inicializamos en el nodo 1
var START_NODE = 1;
//Manejadores
var newSessionHandler = {
  //Inicia el juego poniendo el estado en STARTMODE cuando no escucha intent específico y pregunta si está listo para iniciar
  'LaunchRequest': function () {
    this.handler.state = states.STARTMODE;
    this.emit(':ask', welcomeMessage, repeatWelcomeMessage);
  },
  //Si escucha HELP manda el mensaje de ayuda 
  'AMAZON.HelpIntent': function () {
    this.handler.state = states.STARTMODE;
    this.emit(':ask', helpMessage, helpMessage);
  },
  //Si escucha un intent no conocido repite la pregunta inicial
  'Unhandled': function () {
    this.handler.state = states.STARTMODE;
    this.emit(':ask', promptToStartMessage, promptToStartMessage);
  }
};

//Funciones que controlan los estados
//STARTMODE 
var startGameHandlers = Alexa.CreateStateHandler(states.STARTMODE, {
//Si la respuesta a la pregunta inicial fue Yes, cambia el estado a ASKMODE y hace la primera pregunta
    'AMAZON.YesIntent': function () {
        this.handler.state = states.ASKMODE;
        //Obtiene la pregunta del nodo inicial
        var message = helper.getSpeechForNode(START_NODE);

        //Actualiza el nodo actual
        this.attributes.currentNode = START_NODE;

        //Hace la primera pregunta
        this.emit(':ask', message, message);
    },
    'AMAZON.NoIntent': function () {
        //Si la respuesta es No a la pregunta inicial, emite el mensaje de despedida
        this.emit(':tell', goodbyeMessage);
    },
    'AMAZON.StopIntent': function () {
    //Si la respuesta es Stop a la pregunta inicial, emite el mensaje de despedida
        this.emit(':tell', goodbyeMessage);
    },
    'AMAZON.CancelIntent': function () {
    //Si la respuesta es Cancel a la pregunta inicial, emite el mensaje de despedida
        this.emit(':tell', goodbyeMessage);
    },
    'AMAZON.StartOverIntent': function () {
    //Si la respuesta es StartOver a la pregunta inicial, emite el mensaje inicial de nuevo
         this.emit(':ask', promptToStartMessage, promptToStartMessage);
    },
    'AMAZON.HelpIntent': function () {
    //Si la respuesta es Help a la pregunta inicial, emite el mensaje de ayuda
        this.emit(':ask', helpMessage, helpMessage);
    },
    'Unhandled': function () {
    //Cuando escucha un intent no conocido emite el mensaje inicial de nuevo
        this.emit(':ask', promptToStartMessage, promptToStartMessage);
    }
});
//ASKMODE
//Al usuario se le habrá hecho una pregunta cuando este Intent sea llamado, buscamos ver su respuesta y luego preguntar la siguiente pregunta
//Una vez contestada la última pregunta se le da al usuario el mensaje del nodo final al que llegó y se le pregunta si desea jugar de nuevo
var askQuestionHandlers = Alexa.CreateStateHandler(states.ASKMODE, {

    'AMAZON.YesIntent': function () {
        //Si escucha un Yes, ponemos la respuesta como Yes en la habilidad
        helper.yesOrNo(this,'yes');
    },
    'AMAZON.NoIntent': function () {
        //Si escucha un No, ponemos la respuesta como No en la habilidad
         helper.yesOrNo(this, 'no');
    },
    'AMAZON.StopIntent': function () {
    //Si la respuesta es Stop a la pregunta inicial, emite el mensaje de despedida
        this.emit(':tell', goodbyeMessage);
    },
    'AMAZON.CancelIntent': function () {
    //Si la respuesta es Cancel a la pregunta inicial, emite el mensaje de despedida
        this.emit(':tell', goodbyeMessage);
    },
    'AMAZON.StartOverIntent': function () {
    //Si la respuesta es StartOver a la pregunta inicial, emite el mensaje inicial de nuevo
         this.emit(':ask', promptToStartMessage, promptToStartMessage);
    },
    'AMAZON.HelpIntent': function () {
    //Si la respuesta es Help a la pregunta inicial, emite el mensaje de ayuda
        this.emit(':ask', helpMessage, helpMessage);
    },
    'Unhandled': function () {
    //Cuando escucha un intent no conocido emite el mensaje inicial de nuevo
        this.emit(':ask', promptToStartMessage, promptToStartMessage);
    }
});

var restartHandlers = Alexa.CreateStateHandler(states.RESTARTMODE, {

 'AMAZON.YesIntent': function () {
        // Handle Yes intent.
        // reset the game state to start mode
        this.handler.state = states.STARTMODE;
        this.emit(':ask', welcomeMessage, repeatWelcomeMessage);
    },
    'AMAZON.NoIntent': function () {
        // Handle No intent.
        this.emit(':tell', goodbyeMessage);
    },
    'AMAZON.HelpIntent': function () {
        this.emit(':ask', promptToSayYesNo, promptToSayYesNo);
    },
    'AMAZON.StopIntent': function () {
        this.emit(':tell', goodbyeMessage);
    },
    'AMAZON.CancelIntent': function () {
        this.emit(':tell', goodbyeMessage);
    },
    'AMAZON.StartOverIntent': function () {
        // reset the game state to start mode
        this.handler.state = states.STARTMODE;
        this.emit(':ask', welcomeMessage, repeatWelcomeMessage);
    },
    'Unhandled': function () {
        this.emit(':ask', promptToSayYesNo, promptToSayYesNo);
    }
});

//Funciones de ayuda no ligadas a estados específicos
var helper = {

    //Contiene la lógica para saber que nodo sigue de acuerdo a la respuesta escuchada
    yesOrNo: function (context, reply) {

        //Se revisa si la respuesta fue Yes o No y se obtiene el siguiente nodo de acuerdo a esa respuesta
        var nextNodeId = helper.getNextNode(context.attributes.currentNode, reply);

        //Manejo de errores
        if (nextNodeId == -1)
        {
            context.handler.state = states.STARTMODE;

            //Se manda el mensaje de que el nodo no fue encontrado
            context.emit(':tell', nodeNotFoundMessage, nodeNotFoundMessage);
        }

        //Se obtiene el mensaje del siguiente nodo pregunta
        var message = helper.getSpeechForNode(nextNodeId);

        //Revisa si el siguiente nodo es un nodo final
        if (helper.isAnswerNode(nextNodeId) === true) {

			 // set the game state to description mode
            context.handler.state = states.RESTARTMODE;
            //Se crea un mensaje que contiene el resultado final y la pregunta si se desea jugar de nuevo
            message = message + ', ' + playAgainMessage;
        }

        //Si no es un nodo final, cambia el nodo actual por el nodo siguiente 
        context.attributes.currentNode = nextNodeId;
	//Se emite la siguiente pregunta
        context.emit(':ask', message, message);
    },

    //Esta función obtiene el mensaje del nodo pregunta proporcionado
    getSpeechForNode: function (nodeId) {
        for (var i = 0; i < nodes.length; i++) {
            if (nodes[i].node == nodeId) {
                return nodes[i].message;
            }
        }
        return speechNotFoundMessage + nodeId;
    },

    //Revisa si el nodo proporcionado es nodo pregunta o nodo final
    isAnswerNode: function (nodeId) {
        for (var i = 0; i < nodes.length; i++) {
            if (nodes[i].node == nodeId) {
                if (nodes[i].yes === 0 && nodes[i].no === 0) {
                    return true;
                }
            }
        }
        return false;
    },

    //Obtiene el siguiente nodo de acuerdo a si la respuesta fue Yes o No
    getNextNode: function (nodeId, yesNo) {
        for (var i = 0; i < nodes.length; i++) {
            if (nodes[i].node == nodeId) {
                if (yesNo == "yes") {
                    return nodes[i].yes;
                }
                return nodes[i].no;
            }
        }
        //Regresa un -1 si no encuentra el nodo
        return -1;
    }
};

//Inicializa la habilidad
exports.handler = function (event, context, callback) {
    var alexa = Alexa.handler(event, context);
    alexa.registerHandlers(newSessionHandler, startGameHandlers, askQuestionHandlers, restartHandlers);
    alexa.execute();
};