'use strict';
//Variables globales
var Alexa = require("alexa-sdk");
var states = {
    GUESSMODE: '_GUESSMODE', // User is trying to guess the number.
    STARTMODE: '_STARTMODE'  // Prompt the user to start or restart the game.
};
//Manejadores
var newSessionHandlers = {
	//NewSession es para capturar cualquier forma de iniciar el juego, con o sin intent
	//Revisa cuantas veces ha sido jugado y si es la primera vez inicializa el contador
	//Pone el estado en STARTMODE, emite las veces que se ha jugado y pregunta si desea iniciar
    'NewSession': function() {
        if(Object.keys(this.attributes).length === 0) {
            this.attributes['endedSessionCount'] = 0;
            this.attributes['gamesPlayed'] = 0;
        }
        this.handler.state = states.STARTMODE;
        this.emit(':ask', 'Welcome to High Low guessing game. You have played '
            + this.attributes['gamesPlayed'].toString() + ' times. would you like to play?',
            'Say yes to start the game or no to quit.');
    },
    //Emite mensaje de despedida cuando escucha el intent Stop
    "AMAZON.StopIntent": function() {
      this.emit(':tell', "Goodbye!");  
    },
    //Emite mensaje de despedida cuando escucha el intent Cancel
    "AMAZON.CancelIntent": function() {
      this.emit(':tell', "Goodbye!");  
    },
    //Cuando se termina la sesion, aumenta el contador de veces jugadas
    'SessionEndedRequest': function () {
        this.attributes['endedSessionCount'] += 1;
        this.emit(":tell", "Goodbye!");
    }
};
//Manejadores para el estado STARTMODE
var startGameHandlers = Alexa.CreateStateHandler(states.STARTMODE, {
    'NewSession': function () {
        this.emit('NewSession'); // Uses the handler in newSessionHandlers
    },
    //Emite el mensaje de ayuda cuando escucha el intent Help
    'AMAZON.HelpIntent': function() {
        var message = 'I will think of a number between zero and one hundred, try to guess and I will tell you if it' +
            ' is higher or lower. Do you want to start the game?';
        this.emit(':ask', message, message);
    },
    //Cuando escucha el intent Yes para la pregunta inicial, genera un número al azar, pide al usuario que intente adivinar y cambia el estado a GUESSMODE.
    'AMAZON.YesIntent': function() {
        this.attributes["secretNumber"] = Math.floor(Math.random() * 100);
        this.handler.state = states.GUESSMODE;
        this.emit(':ask', 'Great! ' + 'Try saying a number to start the game.', 'Try saying a number.');
    },
    //Cuando escucha el intent No para la pregunta inicial, manda un mensaje de despedida
    'AMAZON.NoIntent': function() {
        this.emit(':tell', 'Ok, see you next time!');
    },
    //Emite mensaje de despedida cuando escucha el intent Stop
    "AMAZON.StopIntent": function() {
      this.emit(':tell', "Goodbye!");  
    },
    //Emite mensaje de despedida cuando escucha el intent Cancel
    "AMAZON.CancelIntent": function() {
      this.emit(':tell', "Goodbye!");  
    },
    //Cuando se termina la sesion, aumenta el contador de veces jugadas
    'SessionEndedRequest': function () {
        this.attributes['endedSessionCount'] += 1;
        this.emit(':tell', "Goodbye!");
    },
    //Cuando escucha un intent no conocido emite el mensaje de ayuda
    'Unhandled': function() {
        var message = 'Say yes to continue, or no to end the game.';
        this.emit(':ask', message, message);
    }
});
//Manejadores para GUESSMODE
var guessModeHandlers = Alexa.CreateStateHandler(states.GUESSMODE, {
    'NewSession': function () {
        this.handler.state = '';
        this.emitWithState('NewSession'); // Equivalent to the Start Mode NewSession handler
    },
    //Cada que el usuario mencione un número, obtiene el valor númerico gracias al Slot Type y lo compara con el valor generado al azar
    'NumberGuessIntent': function() {
        var guessNum = parseInt(this.event.request.intent.slots.number.value);
        var targetNum = this.attributes["secretNumber"];
	//Si el número mencionado es mayor que el número a adivinar se le dice al usuario
        if(guessNum > targetNum){
            this.emit('TooHigh', guessNum);
        //Si el número mencionado es menor que el número a adivinar se le dice al usuario
        } else if( guessNum < targetNum){
            this.emit('TooLow', guessNum);
        //Si el número mencionado es igual que el número a adivinar se le dice al usuario
        } else if (guessNum === targetNum){
            // Se usa la función flecha para usar una función manteniendo el contexto actual. De otra manera cambiaría a StartMode y sería otro contexto
            this.emit('JustRight', () => {
                this.emit(':ask', guessNum.toString() + ' is correct! Would you like to play a new game?',
                'Say yes to start a new game, or no to end the game.');
        })
        } else {
            this.emit('NotANum');
        }
    },
    //Emite mensaje de ayuda cuando escucha el intent Help
    'AMAZON.HelpIntent': function() {
        this.emit(':ask', 'I am thinking of a number between zero and one hundred, try to guess and I will tell you' +
            ' if it is higher or lower.', 'Try saying a number.');
    },
    //Emite mensaje de despedida cuando escucha el intent Stop
    "AMAZON.StopIntent": function() {
      this.emit(':tell', "Goodbye!");  
    },
    //Emite mensaje de despedida cuando escucha el intent Cancel
    "AMAZON.CancelIntent": function() {
      this.emit(':tell', "Goodbye!");
    },
    //Cuando se termina la sesion, aumenta el contador de veces jugadas
    'SessionEndedRequest': function () {
        this.attributes['endedSessionCount'] += 1;
        this.emit(':tell', "Goodbye!");
    },
    //Cuando escucha un intent no conocido emite un mensaje de ayuda
    'Unhandled': function() {
        this.emit(':ask', 'Sorry, I didn\'t get that. Try saying a number.', 'Try saying a number.');
    }
});

//Funciones de ayuda no ligadas a estados específicos
var helper = {
    //Emite el mensaje de que el número es muy alto
    'TooHigh': function(val) {
        this.emit(':ask', val.toString() + ' is too high.', 'Try saying a smaller number.');
    },
    //Emite el mensaje de que el número es muy bajo
    'TooLow': function(val) {
        this.emit(':ask', val.toString() + ' is too low.', 'Try saying a larger number.');
    },
    //Cambia el estado a STARTMODE y aumenta la cantidad de veces jugado
    'JustRight': function(callback) {
        this.handler.state = states.STARTMODE;
        this.attributes['gamesPlayed']++;
        callback();
    },
    //Emite mensaje de ayuda cuando el usuario dice algo que no sea un número
    'NotANum': function() {
        this.emit(':ask', 'Sorry, I didn\'t get that. Try saying a number.', 'Try saying a number.');
    }
};

//Inicializa la habilidad
exports.handler = function(event, context, callback) {
    var alexa = Alexa.handler(event, context);
    alexa.dynamoDBTableName = 'highLowRecords';
    alexa.registerHandlers(newSessionHandlers, guessModeHandlers, startGameHandlers, helper);
    alexa.execute();
};