#!/usr/bin/env python

import RPi.GPIO as GPIO
import time
import os
import json
from AWSIoTPythonSDK.MQTTLib import AWSIoTMQTTShadowClient

class OnOff:
    def __init__(self, name, iot):
        self.name = name

        self.shadow = iot.createShadowHandlerWithName(self.name, True)
        self.shadow.shadowRegisterDeltaCallback(self.newShadow)
        self.set(False)

    def set(self, state):
        GPIO.setmode(GPIO.BCM)
        GPIO.setwarnings(False)
        GPIO.setup(23,GPIO.OUT)
        if state:
            print "Device on"
            GPIO.output(23,GPIO.HIGH)
            time.sleep(1)
        else:
            print "Device off"
            GPIO.output(23,GPIO.LOW)

        self.shadow.shadowUpdate(json.dumps({
            'state': {
                'reported': {
                    'light': state
                    }
                }
            }
        ), None, 5)

    def newShadow(self, payload, responseStatus, token):
        newState = json.loads(payload)['state']['light']
        self.set(newState)

def createIoT():
    iot = AWSIoTMQTTShadowClient('RelayIOT', useWebsocket=True)
    iot.configureEndpoint('*Endpoint de las Things*', 443)
    iot.configureCredentials(os.path.join(os.path.dirname(os.path.realpath(__file__)), 'VeriSign-Class 3-Public-Primary-Certification-Authority-G5.pem'))
    iot.configureConnectDisconnectTimeout(10)  # 10 sec
    iot.configureMQTTOperationTimeout(5)  # 5 sec
    iot.connect()
    return iot

if __name__ == "__main__":
    iot = createIoT()

    OnOff('table-lamp',iot)

    print('Listening...')

    while True:
        time.sleep(0.2)
