#Imports para las funciones
from __future__ import print_function
import json
import boto3
import time

#El APP_ID de la aplicación (se consigue desde developer.amazon)
APP_ID = "amzn1.ask.skill.d8c4331b-3777-464c-bf98-fa8ae225a305"

#Los intents definidos para la habilidad
TV_INTENTS = [
    "VolumeUp",
    "VolumeDown",
    "VolumeChange",
	"Power",
	"KidsChannel"
]

#Mapas de interación para enviar según el intent
INTENT_ACTION_MAP = {
    "VolumeUp": "VOL_UP",
    "VolumeDown": "VOL_DOWN",
	"Power": "POWER"
}

TV_KIDS_MAP={
	"funny":"KFUNNY",
	"educative":"KEDUC",
	"movies":"KMOVIE"
}

TV_COMMAND_MAP = {
    "up": "VOL_UP",
    "loud": "VOL_UP5",
    "louder": "VOL_UP10",
    "down": "VOL_DOWN",
    "quiet": "VOL_DOWN5",
    "quieter": "VOL_DOWN10"
}



iot_client = boto3.client('iot-data', region_name='us-east-1')

def lambda_handler(event, context):
    """ Ruta que va a tomar el pedido basado en el tipo (LaunchRequest, IntentRequest,
    etc.).
    """
    print("event.session.application.applicationId=" +
          event['session']['application']['applicationId'])
    print(event)

    # chequeo de id de la aplicación
    if (event['session']['application']['applicationId'] != APP_ID):
        raise ValueError("Invalid Application ID")
    
    #Si es un intent regresar la petición
    if event['request']['type'] == "IntentRequest":
        return on_intent(event['request'])
    else:
        return create_help_response()


def on_intent(intent_request):
    print("on_intent")

    # Dispatch skill's intent handlers
    print("entra 1")
    print(intent_request['intent']['name'])
    if intent_request['intent']['name'] in TV_INTENTS:
        print("entra 2")
        print(intent_request['intent'])
        if tv_intent(intent_request['intent']):
            return create_ok_response()
    return create_help_response()

def tv_intent(intent):
    if intent['name'] == "VolumeUp" or intent['name'] == "VolumeDown" or intent['name'] == "Power":
        return send_command(INTENT_ACTION_MAP[intent['name']])
    elif intent['name'] == "VolumeChange":
        action_type = intent["slots"]["action"]["value"]
        if action_type in TV_COMMAND_MAP.keys():
            return send_command(TV_COMMAND_MAP[action_type])
    elif intent['name'] == "KidsChannel":
        action_type = intent["slots"]["channel"]["value"]
        if action_type in TV_KIDS_MAP.keys():
            return send_command(TV_KIDS_MAP[action_type])
    return False

# --------------- Funciones que controlan el comportamiento de la habilidad ------------------

def create_help_response():
    """ build help message """

    help_message = """You can control tv volume, you can
    say tell turn volume up for volume control
    or tell pi home change tv volume loud to increase it 5 times"""
    return build_response(build_speechlet_response(
        help_message, help_message))

def create_ok_response():
    """ build OK message"""
    return build_response(build_speechlet_response(
        "OK", "OK"))

# ---Constructor para las respuestas---

def build_speechlet_response(output, reprompt_text):
    return {
        'outputSpeech': {
            'type': 'PlainText',
            'text': output
        },
        'reprompt': {
            'outputSpeech': {
                'type': 'PlainText',
                'text': reprompt_text
            }
        },
        'shouldEndSession': True
    }


def build_response(speechlet_response):
    return {
        'version': '1.0',
        'response': speechlet_response
    }


def send_command(command):
    shadow = {
        'state': {
            'desired': {
                'command': command,
                'counter': int(time.time())
            }
        }
    }
    payload = json.dumps(shadow)

    response = iot_client.update_thing_shadow(
        thingName='ControllerPhilips',
        payload= payload
    )
    return True

def describe_current_command():
    response = iot_client.get_thing_shadow(
        thingName='ControllerPhilips'
    )
    streamingBody = response["payload"]
    jsonState = json.loads(streamingBody.read())
    print(jsonState)
    return jsonState['state']['desired']['command']